searchNodes=[{"doc":"A task responsible for downloading the precompiled NIFs for a given module. This task must only be used by package creators who want to ship the precompiled NIFs. The goal is to download the precompiled packages and generate a checksum to check-in alongside the project in the the Hex repository. This is done by passing the --all flag. You can also use the --only-local flag to download only the precompiled package for use during development. You can use the --ignore-unavailable flag to ignore any NIFs that are not available. This is useful when you are developing a new NIF that does not support all platforms. This task also accept the --print flag to print the checksums.","ref":"Mix.Tasks.RustlerPrecompiled.Download.html","title":"Mix.Tasks.RustlerPrecompiled.Download","type":"task"},{"doc":"Download and use precompiled NIFs safely with checksums. Rustler Precompiled is a tool for library maintainers that rely on Rustler. It helps by removing the need to have the Rust compiler installed in the user's machine. Check the Precompilation Guide for details. Example defmodule MyNative do use RustlerPrecompiled , otp_app : :my_app , crate : &quot;my_app_nif&quot; , base_url : &quot;https://github.com/me/my_project/releases/download/v0.1.0&quot; , version : &quot;0.1.0&quot; end Options :otp_app - The OTP app name that the dynamic library will be loaded from. :crate - The name of Rust crate if different from the :otp_app . This is optional. :base_url - A valid URL that is used as base path for the NIF file. :version - The version of precompiled assets (it is part of the NIF filename). :force_build - Force the build with Rustler . This is false by default, but if your :version is a pre-release (like &quot;2.1.0-dev&quot;), this option will always be set true . You can also configure this option by setting an application env like this: config :rustler_precompiled , :force_build , your_otp_app : true In case &quot;force build&quot; is used, all options except :base_url , :version and :force_build are going to be passed down to Rustler . So if you need to configure the build, check the Rustler options.","ref":"RustlerPrecompiled.html","title":"RustlerPrecompiled","type":"module"},{"doc":"Returns URLs for NIFs based on its module name. The module name is the one that defined the NIF and this information is stored in a metadata file.","ref":"RustlerPrecompiled.html#available_nif_urls/1","title":"RustlerPrecompiled.available_nif_urls/1","type":"function"},{"doc":"List all default available targets.","ref":"RustlerPrecompiled.html#available_targets/0","title":"RustlerPrecompiled.available_targets/0","type":"function"},{"doc":"Returns the file URL to be downloaded for current target. It receives the NIF module.","ref":"RustlerPrecompiled.html#current_target_nif_url/1","title":"RustlerPrecompiled.current_target_nif_url/1","type":"function"},{"doc":"Returns the target triple for download or compile and load. This function is translating and adding more info to the system architecture returned by Elixir/Erlang to one used by Rust. The returned string has the following format: &quot;nif-NIF_VERSION-ARCHITECTURE-VENDOR-OS-ABI&quot; Examples iex&gt; RustlerPrecompiled . target ( ) { :ok , &quot;nif-2.16-x86_64-unknown-linux-gnu&quot; } iex &gt; RustlerPrecompiled . target ( ) { :ok , &quot;nif-2.15-aarch64-apple-darwin&quot; }","ref":"RustlerPrecompiled.html#target/1","title":"RustlerPrecompiled.target/1","type":"function"},{"doc":"Rustler provides an easy way to use safer NIFs in OTP applications. But in some environments it's harder to use the benefits of the tool because every user needs to install the Rust toolchain and compile the project, which can take several minutes in some cases. This changes with the help of the RustlerPrecompiled package. Now we can easily use precompiled Rustler NIFs from an external source. The precompilation happens in a CI server, always in a transparent way, and the Hex package published should always include a checksum file to ensure the NIFs stays the same, therefore avoiding supply chain attacks. In this guide I will show you how to prepare your project to use this feature.","ref":"precompilation_guide.html","title":"Precompilation guide","type":"extras"},{"doc":"Most of the work is done in the CI server. In this example we are going to use GitHub Actions. The GH Actions service has the benefit of hosting artifacts for releases and make them public available. Usually we want to build for the most popular targets and the three last NIF versions. NIF versions are more stable than OTP versions because they only change after two major releases of OTP. For this guide our targets will be the following: OS: Linux, Windows, macOS Architectures: x86_64 , aarch64 (ARM 64 bits), arm NIF versions: 2.14 , 2.15 , 2.16 . In summary the build matrix looks like this: matrix: job: # NIF version 2.16 - { target: arm-unknown-linux-gnueabihf , os: ubuntu-20.04 , nif: &quot;2.16&quot;, use-cross: true } - { target: aarch64-unknown-linux-gnu , os: ubuntu-20.04 , nif: &quot;2.16&quot;, use-cross: true } - { target: aarch64-apple-darwin , os: macos-10.15 , nif: &quot;2.16&quot; } - { target: x86_64-apple-darwin , os: macos-10.15 , nif: &quot;2.16&quot; } - { target: x86_64-unknown-linux-gnu , os: ubuntu-20.04 , nif: &quot;2.16&quot; } - { target: x86_64-unknown-linux-musl , os: ubuntu-20.04 , nif: &quot;2.16&quot;, use-cross: true } - { target: x86_64-pc-windows-gnu , os: windows-2019 , nif: &quot;2.16&quot; } - { target: x86_64-pc-windows-msvc , os: windows-2019 , nif: &quot;2.16&quot; } # NIF version 2.15 - { target: arm-unknown-linux-gnueabihf , os: ubuntu-20.04 , nif: &quot;2.15&quot;, use-cross: true } - { target: aarch64-unknown-linux-gnu , os: ubuntu-20.04 , nif: &quot;2.15&quot;, use-cross: true } - { target: aarch64-apple-darwin , os: macos-10.15 , nif: &quot;2.15&quot; } - { target: x86_64-apple-darwin , os: macos-10.15 , nif: &quot;2.15&quot; } - { target: x86_64-unknown-linux-gnu , os: ubuntu-20.04 , nif: &quot;2.15&quot; } - { target: x86_64-unknown-linux-musl , os: ubuntu-20.04 , nif: &quot;2.15&quot;, use-cross: true } - { target: x86_64-pc-windows-gnu , os: windows-2019 , nif: &quot;2.15&quot; } - { target: x86_64-pc-windows-msvc , os: windows-2019 , nif: &quot;2.15&quot; } # NIF version 2.14 - { target: arm-unknown-linux-gnueabihf , os: ubuntu-20.04 , nif: &quot;2.14&quot;, use-cross: true } - { target: aarch64-unknown-linux-gnu , os: ubuntu-20.04 , nif: &quot;2.14&quot;, use-cross: true } - { target: aarch64-apple-darwin , os: macos-10.15 , nif: &quot;2.14&quot; } - { target: x86_64-apple-darwin , os: macos-10.15 , nif: &quot;2.14&quot; } - { target: x86_64-unknown-linux-gnu , os: ubuntu-20.04 , nif: &quot;2.14&quot; } - { target: x86_64-unknown-linux-musl , os: ubuntu-20.04 , nif: &quot;2.14&quot;, use-cross: true } - { target: x86_64-pc-windows-gnu , os: windows-2019 , nif: &quot;2.14&quot; } - { target: x86_64-pc-windows-msvc , os: windows-2019 , nif: &quot;2.14&quot; } A complete workflow example can be found in the rustler_precompilation_example project.","ref":"precompilation_guide.html#prepare-for-the-build","title":"Precompilation guide - Prepare for the build","type":"extras"},{"doc":"In our build we are going to cross compile our crate project (the Rust code for our NIF) using a variety of targets as we saw in the previous section. For this to work we need to guide the Rust compiler in some cases by providing additional configuration in the .cargo/config file of our project. Here is an example of that file: [target.'cfg(target_os = &quot;macos&quot;)'] rustflags = [ &quot;-C&quot;, &quot;link-arg=-undefined&quot;, &quot;-C&quot;, &quot;link-arg=dynamic_lookup&quot;, ] # See https://github.com/rust-lang/rust/issues/59302 [target.x86_64-unknown-linux-musl] rustflags = [ &quot;-C&quot;, &quot;target-feature=-crt-static&quot; ] # Provides a small build size, but takes more time to build. [profile.release] lto = true In addition to that, we also use a tool called cross that makes the build easier for some targets (the ones using use-cross: true in our example).","ref":"precompilation_guide.html#additional-configuration-before-build","title":"Precompilation guide - Additional configuration before build","type":"extras"},{"doc":"We need to tell RustlerPrecompiled where to find our NIF files, and we need to tell which version to use. defmodule RustlerPrecompilationExample.Native do version = Mix.Project . config ( ) [ :version ] use RustlerPrecompiled , otp_app : :rustler_precompilation_example , crate : &quot;example&quot; , base_url : &quot;https://github.com/philss/rustler_precompilation_example/releases/download/v \#{ version } &quot; , force_build : System . get_env ( &quot;RUSTLER_PRECOMPILATION_EXAMPLE_BUILD&quot; ) in [ &quot;1&quot; , &quot;true&quot; ] , version : version # When your NIF is loaded, it will override this function. def add ( _a , _b ) , do : :erlang . nif_error ( :nif_not_loaded ) end This example was extracted from the rustler_precompilation_example project. RustlerPrecompiled will try to figure out the target and download the correct file for us. This will happen in compile time only. Optionally it's possible to force the compilation by setting an env var, like the example suggests. It's also possible to force the build by using a pre release version, like 0.1.0-dev .","ref":"precompilation_guide.html#the-rustler-module","title":"Precompilation guide - The Rustler module","type":"extras"},{"doc":"In a scenario where you need to release a Hex package using precompiled NIFs, you first need to build the release in the CI, wait for all artifacts to be available and then generate the checksum file that is MANDATORY for your package to work. This checksum file is generated by running the following command after the build is complete: $ mix rustler_precompiled.download YourRustlerModule --all --print With the module I used for this guide, the command would be: $ mix rustler_precompiled.download RustlerPrecompilationExample.Native --all --print The file generated will be named checksum-Elixir.RustlerPrecompilationExample.Native.exs and it's extremely important that you include this file in your Hex package (by updating the files: field in your mix.exs ). Otherwise your package won't work . You don't need to track this file in your version control system (git or other). To recap, the suggested flow is the following: release a new tag push the code to your repository with the new tag: git push origin main --tags wait for all NIFs to be built run the mix rustler_precompiled.download task (with the flag --all ) release the package to Hex.pm.","ref":"precompilation_guide.html#the-release-flow","title":"Precompilation guide - The release flow","type":"extras"},{"doc":"The ability to use precompiled NIFs written in Rust can increase the adoption of some packages, because people won't need to have Rust installed. But this comes with some drawbacks and more responsibilities to the maintainers, so use this feature carefully.","ref":"precompilation_guide.html#conclusion","title":"Precompilation guide - Conclusion","type":"extras"},{"doc":"All notable changes to this project will be documented in this file. The format is based on Keep a Changelog , and this project adheres to Semantic Versioning .","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#unreleased","title":"Changelog - Unreleased","type":"extras"},{"doc":"Added Add the possibility to skip the download of unavailable NIFs when generating the checksum file - thanks @fahchen","ref":"changelog.html#0-3-0-2022-03-26","title":"Changelog - 0.3.0 - 2022-03-26","type":"extras"},{"doc":"Fixed Fix validation of URL in order to be compatible with Elixir ~&gt; 1.11. The previous implementation was restricted to Elixir ~&gt; 1.13. Added Add :force_build option that fallback to Rustler . It passes all options except the ones used by RustlerPrecompiled down to Rustler . This option will be by default false , but if the project is using a pre-release, then it will always be set to true . With this change the project starts depending on Rustler. Changed Relax dependencies to the minor versions.","ref":"changelog.html#0-2-0-2022-02-18","title":"Changelog - 0.2.0 - 2022-02-18","type":"extras"},{"doc":"Added Add basic features to download and use the precompiled NIFs in a safe way.","ref":"changelog.html#0-1-0-2022-02-16","title":"Changelog - 0.1.0 - 2022-02-16","type":"extras"}]